
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.util.UIUtils;



/*
 * Represents the integer
 */
public class IntegerAttribute extends Attribute {
    private int defaultValue;

    public IntegerAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();
        setValue(0);
    }

    public IntegerAttribute(String name, int defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public IntegerAttribute(String name, int defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public int getType() {
        return AttributeTypes.INTEGER;
    }

    public int getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return UIUtils.getIntValue(valueList.get(0));
    }

    public int[] getValues() {
        return UIUtils.getIntArray(valueList);
    }

    public void setValue(int value) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(new Integer(value));
        } else {
            valueList.add(0, new Integer(value));
        }
    }

    public void setValue(int occurrence, int value) {
        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(occurrence, new Integer(value));
        } else {
            valueList.add(0, new Integer(value));
        }
    }

    public void setValues(int[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(new Integer(values[i]));
            }
        } else {
            valueList.add(0, new Integer(values[0]));
        }
    }

    public int getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(int defValue) {
        defaultValue = defValue;
    }

    public void printMe() {
        super.printMe();
        System.out.println("\t\tDefVal:" + defaultValue);
    }
}
