
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.dataobjects.attributes;

import com.ibm.ccd.ui.util.UIUtils;


/*
 * Represents the Number attribute for the real types(float/double)
 */
public class NumberAttribute extends Attribute {
    //	def. value
    private double defaultValue;

    // precision
    private int precision = 2;

    public NumberAttribute(String name, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        removeValues();
        setValue(defaultValue);
    }

    public NumberAttribute(String name, boolean editable, int precision,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.precision = precision;
        removeValues();
        setValue(defaultValue);
        this.precision = precision;
    }

    public NumberAttribute(String name, double defaultValue, boolean editable,
        int minimumOccurrence, int maximumOccurrence, String displayName) {
        super(name, editable, minimumOccurrence, maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public NumberAttribute(String name, double defaultValue, boolean editable,
        int minimumLength, int maximumLength, int minimumOccurrence,
        int maximumOccurrence, String displayName) {
        super(name, editable, minimumLength, maximumLength, minimumOccurrence,
            maximumOccurrence, displayName);
        this.defaultValue = defaultValue;
        removeValues();
        setValue(defaultValue);
    }

    public int getType() {
        return AttributeTypes.NUMBER;
    }

    public double getValue() {
        if ((getValueList() == null) || (getValueList().size() == 0)) {
            throw new RuntimeException("Value not set for " + getName());
        }

        return UIUtils.getDoubleValue(valueList.get(0));
    }

    public double[] getValues() {
        return UIUtils.getDoubleArray(valueList);
    }

    public void setValue(double value) {
        if (getCurrentOccurrence() > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(new Double(value));
        } else {
            valueList.add(0, new Double(value));
        }
    }

    public void setValue(int occurrence, double value) {
        if (occurrence > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.add(occurrence, new Double(value));
        } else {
            valueList.add(0, new Double(value));
        }
    }

    public void setValues(double[] values) {
        if ((values == null) || (values.length == 0)) {
            return;
        }

        if (values.length > getMaximumOccurrence()) {
            throw new RuntimeException("Occurrence is exceeded for " +
                getName());
        }

        if (isMultiValued()) {
            valueList.clear();

            for (int i = 0; i < values.length; ++i) {
                valueList.add(new Double(values[i]));
            }
        } else {
            valueList.add(0, new Double(values[0]));
        }
    }

    public double getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(double defValue) {
        defaultValue = defValue;
    }

    public int getPrecision() {
        return precision;
    }

    public void setPrecision(int i) {
        precision = i;
    }

    public void printMe() {
        super.printMe();
        System.out.println("\t\tDef.Val:" + defaultValue);
    }
}
