
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.dataobjects.items;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.jetspeed.portlet.PortletRequest;
import org.w3c.dom.Node;

import com.ibm.ccd.ui.dataobjects.FileUploadResource;
import com.ibm.ccd.ui.dataobjects.NVPair;
import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.AttributeTypes;
import com.ibm.ccd.ui.dataobjects.attributes.BinaryAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.CurrencyAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.FlagAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.GroupAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.IntegerAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.LookupTableAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.NumberEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PasswordAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.PeriodAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.RelationshipAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.StringEnumAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.ThumbnailImageURLAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.TimeZoneAttribute;
import com.ibm.ccd.ui.dataobjects.attributes.URLAttribute;
import com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea;
import com.ibm.ccd.ui.exception.MessageMap;
import com.ibm.ccd.ui.mediator.WPCMediator;
import com.ibm.ccd.ui.portlets.massupdate.IMassUpdate;
import com.ibm.ccd.ui.util.IReference;
import com.ibm.ccd.ui.util.UIUtils;

/**
 *  Represents the WPC Catalog Item/Spec
 *  The CatalogItem must have catalogID and itemID (PrimaryKey) values and composes several Attributes
 */
	
// TODO once the dyanamic rendereing enabled the CatalogItem should be 
// reorganized to accommodate views and configurable JSPS
public class CatalogItem implements ICatalogItem,Serializable {
	// parent category ID
	private String prentCtgyID;

	// catalogID
	private String catalogID;

	// CollabrationArea
	private CollaborationArea collabArea;

	// CatalogVersion
	private String ctgVersion;

	// Item unique indentifier, likely to be the primaryKey of the item
	private String itemID;
	private Attribute primaryKeyAttr = null;

	// List of attributes
	private Attribute[] attributes;

	// Attributes list is re-organized with map being key as the attribute name and value as the attribute
	// which helps in retreiving the attribute based on the name, rather than iterating on the array 
	private HashMap attributesMap;
	private HashMap flatAttributeMap;
	private HashMap downloadAttributeMap;

	// View JSP used to Create (displaying the create form) Item
	private String createJSP;

	//	View JSP used to display the Create success state
	private String createSuccessJSP;

	//	View JSP used to display the Create failure state
	private String createFailureJSP;

	//	View JSP used to Update (displaying the update form) Item
	private String updateJSP;

	//	View JSP used to display the Update success state
	private String updateSuccessJSP;

	//	View JSP used to display the Update failure state
	private String updateFailureJSP;

	//	View JSP used to display the list of items
	private String listJSP;

	//	View JSP used to display the Print view of the item
	private String printJSP;

	public CatalogItem() {
	}

	public CatalogItem(
		String parentCategory,
		String catalogID,
		String catalogVersion,
		Attribute[] attributes) {
		this(catalogID, attributes);
		setParentCategoryID(parentCategory);
		setCatalogVersion(catalogVersion);
		createJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
		updateJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
		printJSP = "/jsps/html/itemcatalog/" + catalogID + "_Create.jsp";
	}

	public CatalogItem(String catalogID, Attribute[] attributes) {
		this.catalogID = catalogID;
		this.attributes = attributes;
		prepareAttributesMap();
		flatAttributeMap = new HashMap();
		flatAttributeMap = prepareFlatAttributeMap(attributes);
	}

	public CatalogItem(
		String parentCategoryID,
		String catalogID,
		String catalogVersion,
		String itemID,
		Attribute[] attributes) {
		this.prentCtgyID = parentCategoryID;
		this.catalogID = catalogID;
		this.itemID = itemID;
		this.attributes = attributes;
		prepareAttributesMap();
		flatAttributeMap = new HashMap();
		flatAttributeMap = prepareFlatAttributeMap(attributes);
	}

	/**
	 * @deprecated
	 */
	public String getCatalogID() {
		if ((catalogID == null) && (catalogID.equals(""))) {
			String catalogIDtemp =
				WPCMediator.getInstance().getDefaultCatalog();

			return catalogIDtemp;
		}

		return catalogID;
	}

	public void setCatalogID(String catalogID) {
		this.catalogID = catalogID;
	}

	public String getItemID() {
		return itemID;
	}

	public void setItemID(String itemID) {
		this.itemID = itemID;
	}

	public String getCreateView() {
		return createJSP;
	}

	public String getCreateSuccessView() {
		return (createSuccessJSP != null) ? createSuccessJSP : createJSP;
	}

	public String getCreateFailureView() {
		return (createFailureJSP != null) ? createFailureJSP : createJSP;
	}

	public void setCreateView(String createJSP) {
		this.createJSP = createJSP;
	}

	public void setCreateSuccessView(String createSuccessJSP) {
		this.createSuccessJSP = createSuccessJSP;
	}

	public void setCreateFailureView(String createFailureJSP) {
		this.createFailureJSP = createFailureJSP;
	}

	public String getUpdateView() {
		return updateJSP;
	}

	public void resetJSP() {
		createJSP = "/jsps/html/itemcatalog/defaultItemCreate.jsp";
		updateJSP = "/jsps/html/itemcatalog/defaultItemCreate.jsp";
		printJSP = "/jsps/html/itemcatalog/defaultItemCreate.jsp";
	}

	public String getUpdateSuccessView() {
		return (updateSuccessJSP != null) ? updateSuccessJSP : updateJSP;
	}

	public String getUpdateFailureView() {
		return (updateFailureJSP != null) ? updateFailureJSP : updateJSP;
	}

	public void setUpdateView(String updateJSP) {
		this.updateJSP = updateJSP;
	}

	public void setUpdateSuccessView(String updateSuccessJSP) {
		this.updateSuccessJSP = updateSuccessJSP;
	}

	public void setUpdateFailureView(String updateFailureJSP) {
		this.updateFailureJSP = updateFailureJSP;
	}

	public String getPrintView() {
		return printJSP;
	}

	public String getListView() {
		return listJSP;
	}

	public void setPrintView(String printJSP) {
		this.printJSP = printJSP;
	}

	public void setListView(String listJSP) {
		this.listJSP = listJSP;
	}

	public void setAttributes(Attribute[] attributes) {
		this.attributes = attributes;
		prepareAttributesMap();
	}

	public Attribute[] getAttributes() {
		return attributes;
	}

	public Attribute getAttribute(String name) {
		return (attributesMap != null)
			? (Attribute) attributesMap.get(name)
			: null;
	}

	/**
	 * The XML parsing capability is externilzed to CatalogItemParser
	 */
	public void fromXML(Node domNode) {
	}

	public void fromXML(String xml) {
	}

	/**
	 * Provides the capability to populate the item properties from the PortletRequest
	 * by iterating on the attributes..
	 */

	public MessageMap setProperty(
		PortletRequest portletRequest,
		String specAttributeName) throws ParseException {
		if (attributes == null) {
			return null;
		}

		
		MessageMap messageMap = (MessageMap) portletRequest.getAttribute(IReference.MESSAGE_KEY);

        if (messageMap == null) {
            messageMap = new MessageMap();
            portletRequest.setAttribute(IReference.MESSAGE_KEY, messageMap);
        }
		
		Attribute attribute =
			(Attribute) flatAttributeMap.get(specAttributeName);
		checkMandataoryAttribute(attribute, portletRequest, messageMap);
		try {
            setAttribProperty(attribute, portletRequest, messageMap);
        } catch (ParseException e) {
            throw e;
        }
		return messageMap;
	}

	protected void setAttribProperty(
		Attribute attribute,
		PortletRequest portletRequest,
		MessageMap messageMap) throws ParseException {
		String massEdit =
			(String) portletRequest.getAttribute(IMassUpdate.KEY_MASS_EDIT);

		// constraints has to be enabled for all the attributes
		String name = "";

		// In case of mass edit item id will be prepended to each name key
		if ((massEdit != null)
			&& (massEdit.equals(IMassUpdate.KEY_MASS_EDIT))) {
			name =
				"ATTR"
					+ itemID
					+ "_"
					+ UIUtils.replaceEmptyStr(
						UIUtils.replaceSlash(attribute.getName()));
		} else {
			name = attribute.getName();
		}

		String pvalue = portletRequest.getParameter(name);

		switch (attribute.getType()) {
			case AttributeTypes.STRING :
				{
					attribute.removeValues();

					StringAttribute stringAttribute =
						(StringAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						values = UIUtils.unescapeForHTML(values);	
						stringAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);
						value = UIUtils.unescapeForHTML(value);
						stringAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.INTEGER :
				{

					IntegerAttribute integerAttribute =
						(IntegerAttribute) attribute;

					if (attribute.isMultiValued()) {

						    String strValues[] = portletRequest.getParameterValues(name);
						    for (int i = 0; i < strValues.length; i++) {
								try {
								    String strValue = strValues[i];
									if (!UIUtils.isEmpty(strValue)) {
									    strValue = UIUtils.parseInteger(strValue,portletRequest.getLocale());
										int value = UIUtils.getIntValue(strValue); 
										integerAttribute.removeValue(i);								
										integerAttribute.setValue(i,value);
									}else{
									    integerAttribute.removeValue(i);								
										integerAttribute.setValue(i,0);
									}
								} catch (ParseException p) {
									throw p;
								}
						    }
					} else {
						try {
						    String strValue = portletRequest.getParameter(name);
							if (!UIUtils.isEmpty(strValue)) {
							    strValue = UIUtils.parseInteger(strValue,portletRequest.getLocale());
								int value = UIUtils.getIntValue(strValue); 
								integerAttribute.removeValue(0);								
								integerAttribute.setValue(0,value);
							}else{
							    integerAttribute.removeValue(0);								
								integerAttribute.setValue(0,0);
							}
						} catch (ParseException p) {
							throw p;
						}
					}

					return;
				}

			case AttributeTypes.NUMBER :
				{
					NumberAttribute numberAttribute =
						(NumberAttribute) attribute;
					if (attribute.isMultiValued()) {
						String[] numvalues =
							portletRequest.getParameterValues(name);					    
						for (int i = 0; i < numvalues.length; i++) {
							try {
                                if (!UIUtils.isEmpty(numvalues[i])) {
                                	String strvalue = UIUtils.parseNumber(numvalues[i],
                                			portletRequest.getLocale());
                                	numberAttribute.removeValue(i);
                                	numberAttribute.setValue(i,
                                		UIUtils.getDoubleValue(strvalue));
                                }else{
                                	numberAttribute.removeValue(i);
                                	numberAttribute.setValue(i,0);
    							}
                            } catch (ParseException p) {
                                throw p;
                            }
						}
					} else {
						String numvalue =
							portletRequest.getParameter(name);
						try {						
						    if (!UIUtils.isEmpty(numvalue)) {
							    numvalue = UIUtils.parseNumber(numvalue,
											portletRequest.getLocale());
							    	double value = UIUtils.getDoubleValue(numvalue);
							    	numberAttribute.removeValue(0);     							    	
									numberAttribute.setValue(0,value);
						    }else{
						    	numberAttribute.removeValue(0);     							    	
								numberAttribute.setValue(0,0);
						    }
						} catch (ParseException p) {
							throw p;
						}
					}
					return;
				}

			case AttributeTypes.DATE :
				{
					attribute.removeValues();

					DateAttribute dateAttribute = (DateAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);

						for (int i = 0; i < values.length; i++) {
							if ((values[i] != null)
								&& (values[i].length() >= 8)
								&& (values[i].length() <= 10)) {
								values[i] = values[i] + " 00:00:00";
							}
						}

						dateAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);

						if ((value != null)
							&& (value.length() >= 8)
							&& (value.length() <= 10)) {
							value = value + " 00:00:00";
						}

						dateAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.FLAG :
				{
					attribute.removeValues();

					FlagAttribute flagAttribute = (FlagAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						String[] setValues = null;

						if ((values != null) && (values.length > 0)) {
							setValues = new String[values.length];

							for (int i = 0; i < values.length; ++i) {
								String val = "false";

								if (values[i] != null) {
									if (values[i].equalsIgnoreCase("on")) {
										val = "true";
									} else {
										val = values[i].toLowerCase();
									}
								}

								setValues[i] = val;
							}
						}

						flagAttribute.setValues(setValues);
					} else {
						String val = "false";
						String paramVal = portletRequest.getParameter(name);

						if (paramVal != null) {
							if (paramVal.equalsIgnoreCase("on")) {
								val = "true";
							} else {
								val = paramVal.toLowerCase();
							}
						}

						flagAttribute.setValue(val);
					}

					return;
				}

			case AttributeTypes.TIMEZONE :
				{
					attribute.removeValues();

					TimeZoneAttribute timeZoneAttribute =
						(TimeZoneAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						timeZoneAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);
						timeZoneAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.CURRENCY :
				{
					attribute.removeValues();

					CurrencyAttribute currencyAttribute =
						(CurrencyAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						values = UIUtils.unescapeForHTML(values);	
						currencyAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);
						value = UIUtils.unescapeForHTML(value);
						currencyAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.PASSWORD :
				{
					attribute.removeValues();

					PasswordAttribute passwordAttribute =
						(PasswordAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						passwordAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);
						passwordAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.PERIOD :
				{
					PeriodAttribute periodAttribute = (PeriodAttribute) attribute;
					periodAttribute.removeValues();
					
					if(periodAttribute.isMultiValued()) {
						for(int i=0;i<attribute.getCurrOccurrence();i++) {
							String startDate = portletRequest.getParameter(name + "_" + i+"_"+"V1");
							String endDate = portletRequest.getParameter(name+ "_" + i+"_"+"V2");
							periodAttribute.setValue(startDate,endDate);
						}
					}else {
						String startDate = portletRequest.getParameter(name + "_" + "0" +"_"+"V1");
						String endDate = portletRequest.getParameter(name + "_" + "0" +"_"+"V2");
						periodAttribute.setValue(startDate,endDate);
					}
					break;
				}
				
			case AttributeTypes.SEQUENCE :
				// sequence value set is not allowed from the user
				// TODO populate message map, if user sets the value
				return;

			case AttributeTypes.RELATIONSHIP :
				{
					attribute.removeValues();

					RelationshipAttribute relationAttrib =
						(RelationshipAttribute) attribute;
					NVPair[] pairs = null;

					String[] values =
						portletRequest.getParameterValues(name + "_1");
					String[] keys =
						portletRequest.getParameterValues(name + "_2");

					pairs = new NVPair[relationAttrib.getMaximumOccurrence()];

					for (int i = 0; i < pairs.length; i++) {
						pairs[i] = new NVPair();
					}

					if ((values != null) && (values.length > 0)) {
						for (int i = 0; i < values.length; i++) {
							pairs[i].setName(values[i]);
						}
					}

					if ((keys != null) && (keys.length > 0)) {
						for (int i = 0; i < keys.length; i++) {
							pairs[i].setValue(keys[i]);
						}
					}

					if (relationAttrib.isMultiValued()) {
						relationAttrib.setValues(pairs);
					} else {
						NVPair pair = new NVPair();

						if ((values != null)
							&& (values.length > 0)
							&& (keys != null)
							&& (keys.length > 0)) {
							relationAttrib.setValue(pairs[0]);
						} else {
							relationAttrib.setValue(pair);
						}
					}

					return;
				}

			case AttributeTypes.BINARY :
				{
					BinaryAttribute binaryAttrib = (BinaryAttribute) attribute;
					FileUploadResource[] files = null;
					String[] binary = portletRequest.getParameterValues(name);

					if ((binary != null) && (binary.length > 0)) {
						if (binaryAttrib.getValues() != null) {
							files = binaryAttrib.getValues();

							for (int i = 0; i < files.length; i++) {
								if ((files[i] != null)
									&& ((files[i].getName() == null)
										|| !files[i].getName().equals(
											binary[i]))) {
									// set server details only when user update with new file attachment
									files[i].setDirectory("");
									files[i].setName("");
									files[i].setPassword("");
									files[i].setServer("");
									files[i].setType("");
									files[i].setUserID("");
									files[i].setName(binary[i]);
								}
							}
						} else {
							files = new FileUploadResource[binary.length];

							for (int i = 0; i < files.length; i++) {
								// set server details for new file attachment 
								files[i] = new FileUploadResource();
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(binary[i]);
							}

							if (binaryAttrib.isMultiValued()) {
								binaryAttrib.setValues(files);
							} else {
								if ((files != null) && (files.length > 0)) {
									binaryAttrib.setValue(files[0]);
								}
							}
						}
					}

					return;
				}

			case AttributeTypes.IMAGE :
				{
					ImageAttribute imgAttrib = (ImageAttribute) attribute;

					FileUploadResource[] files = null;
					String[] images = portletRequest.getParameterValues(name);

					if ((images != null) && (images.length > 0)) {
						if (imgAttrib.getValues() != null) {
							files = imgAttrib.getValues();

							for (int i = 0; i < files.length; i++) {
								if ((files[i] != null)
									&& ((files[i].getName() == null)
										|| !files[i].getName().equals(
											images[i]))) {
									//	set server details only when user update with new file attachment
									files[i].setDirectory("");
									files[i].setName("");
									files[i].setPassword("");
									files[i].setServer("");
									files[i].setType("");
									files[i].setUserID("");
									files[i].setName(images[i]);
								}
							}
						} else {
							files = new FileUploadResource[images.length];

							for (int i = 0; i < images.length; i++) {
								// set server details for new file attachment
								files[i] = new FileUploadResource();
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(images[i]);
							}

							if (imgAttrib.isMultiValued()) {
								imgAttrib.setValues(files);
							} else {
								if ((files != null) && (files.length > 0)) {
									imgAttrib.setValue(files[0]);
								}
							}
						}
					}

					return;
				}

			case AttributeTypes.THUMB_NAIL_IMAGE :
				{
					ThumbnailImageAttribute thumbNailImg =
						(ThumbnailImageAttribute) attribute;
					FileUploadResource[] files = null;
					String[] images = portletRequest.getParameterValues(name);

					if ((images != null) && (images.length > 0)) {
						if (thumbNailImg.getValues() != null) {
							files = thumbNailImg.getValues();

							for (int i = 0; i < files.length; i++) {
								if ((files[i] != null)
									&& ((files[i].getName() == null)
										|| !files[i].getName().equals(
											images[i]))) {
									//set server details only when user update with new file attachment
									files[i].setDirectory("");
									files[i].setName("");
									files[i].setPassword("");
									files[i].setServer("");
									files[i].setType("");
									files[i].setUserID("");
									files[i].setName(images[i]);
								}
							}
						} else {
							files = new FileUploadResource[images.length];

							for (int i = 0; i < images.length; i++) {
								// set server details for new file attachment								
								files[i] = new FileUploadResource();
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(images[i]);
							}

							if (thumbNailImg.isMultiValued()) {
								thumbNailImg.setValues(files);
							} else {
								if ((files != null) && (files.length > 0)) {
									thumbNailImg.setValue(files[0]);
								}
							}
						}
					}

					return;
				}

			case AttributeTypes.URL :
				{
					attribute.removeValues();

					URLAttribute urlAttribute = (URLAttribute) attribute;
					URL[] values = null;

					String[] urlStrings =
						portletRequest.getParameterValues(name);
					urlStrings = UIUtils.unescapeForHTML(urlStrings);	

					if ((urlStrings != null) && (urlStrings.length > 0)) {
						values = new URL[urlStrings.length];

						for (int i = 0; i < urlStrings.length; ++i) {
							try {
								if (!UIUtils.isEmpty(urlStrings[i])) {
									values[i] = new URL(urlStrings[i]);
								}
							} catch (MalformedURLException mue) {
								try {
									values[i] =
										new URL("http", urlStrings[i], "");
								} catch (MalformedURLException urle) {
									mue.printStackTrace();
								}
							}
						}

						if (urlAttribute.isMultiValued()) {
							urlAttribute.setValues(values);
						} else {
							URL value = null;

							if ((values != null) && (values.length > 0)) {
								value = values[0];
							}

							urlAttribute.setValue(value);
						}
					}

					return;
				}

			case AttributeTypes.IMAGE_URL :
				{
					attribute.removeValues();

					ImageURLAttribute imageURLAttribute =
						(ImageURLAttribute) attribute;
					URL[] values = null;

					String[] urlStrings =
						portletRequest.getParameterValues(name);
					urlStrings = UIUtils.unescapeForHTML(urlStrings);

					if ((urlStrings != null) && (urlStrings.length > 0)) {
						values = new URL[urlStrings.length];

						for (int i = 0; i < urlStrings.length; ++i) {
							try {
								if (!UIUtils.isEmpty(urlStrings[i])) {
									values[i] = new URL(urlStrings[i]);
								}
							} catch (MalformedURLException mue) {
								try {
									values[i] =
										new URL("http", urlStrings[i], "");
								} catch (MalformedURLException urle) {
									urle.printStackTrace();
								}
							}
						}

						if (imageURLAttribute.isMultiValued()) {
							imageURLAttribute.setValues(values);
						} else {
							URL value = null;

							if ((values != null) && (values.length > 0)) {
								value = values[0];
							}

							imageURLAttribute.setValue(value);
						}
					}

					return;
				}

			case AttributeTypes.THUMB_NAIL_IMAGE_URL :
				{
					attribute.removeValues();

					ThumbnailImageURLAttribute thumbnailImageURLAttribute =
						(ThumbnailImageURLAttribute) attribute;
					URL[] values = null;

					String[] urlStrings =
						portletRequest.getParameterValues(name);
					urlStrings = UIUtils.unescapeForHTML(urlStrings);
					
					if ((urlStrings != null) && (urlStrings.length > 0)) {
						values = new URL[urlStrings.length];

						for (int i = 0; i < urlStrings.length; ++i) {
							try {
								if (!UIUtils.isEmpty(urlStrings[i])) {
									values[i] = new URL(urlStrings[i]);
								}
							} catch (MalformedURLException mue) {
								try {
									values[i] =
										new URL("http", urlStrings[i], "");
								} catch (MalformedURLException urle) {
									mue.printStackTrace();
								}
							}
						}

						if (thumbnailImageURLAttribute.isMultiValued()) {
							thumbnailImageURLAttribute.setValues(values);
						} else {
							URL value = null;

							if ((values != null) && (values.length > 0)) {
								value = values[0];
							}

							thumbnailImageURLAttribute.setValue(value);
						}
					}

					return;
				}

			case AttributeTypes.STRING_ENUMERATION :
				{
					attribute.removeValues();

					StringEnumAttribute stringEnumAttribute =
						(StringEnumAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						stringEnumAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);

						if (!UIUtils.isEmpty(value)) {
							stringEnumAttribute.setValue(value);
						}
					}

					return;
				}

			case AttributeTypes.NUMBER_ENUMERATION :
				{
					attribute.removeValues();

					NumberEnumAttribute numberEnumAttribute =
						(NumberEnumAttribute) attribute;

					if (attribute.isMultiValued()) {
						try {
							double[] values =
								UIUtils.getDoubleArray(
									portletRequest.getParameterValues(name));
							numberEnumAttribute.setValues(values);
						} catch (Exception e) {
							e.printStackTrace();

							return;
						}
					} else {
						try {
							double value =
								UIUtils.getDoubleValue(
									portletRequest.getParameter(name));
							numberEnumAttribute.setValue(value);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}

					return;
				}

			case AttributeTypes.LOOKUP_TABLE :
				{
					attribute.removeValues();

					LookupTableAttribute lookupTableAttribute =
						(LookupTableAttribute) attribute;

					if (attribute.isMultiValued()) {
						String[] values =
							portletRequest.getParameterValues(name);
						lookupTableAttribute.setValues(values);
					} else {
						String value = portletRequest.getParameter(name);
						lookupTableAttribute.setValue(value);
					}

					return;
				}

			case AttributeTypes.GROUP :
				{
					GroupAttribute groupAttrib = (GroupAttribute) attribute;
					Attribute[] subGroupAttrib = groupAttrib.getAttributes();

					for (int i = 0; i < subGroupAttrib.length; i++) {
						checkMandataoryAttribute(
							subGroupAttrib[i],
							portletRequest,
							messageMap);
						setAttribProperty(
							subGroupAttrib[i],
							portletRequest,
							messageMap);
					}

					return;
				}

				//////////////
		} // end of switch	
	}

	protected void checkMandataoryAttribute(
		Attribute attribute,
		PortletRequest portletRequest,
		MessageMap messageMap) {
		// Mandatory check is not required for Attribute 
		if (attribute.getType() == AttributeTypes.GROUP) {
			return;
		}

		//		String name = attribute.getName();
		String massEdit =
			(String) portletRequest.getAttribute(IMassUpdate.KEY_MASS_EDIT);

		// constraints has to be enabled for all the attributes
		String name = "";

		// In case of mass edit item id will be prepended to each name key
		if ((massEdit != null)
			&& (massEdit.equals(IMassUpdate.KEY_MASS_EDIT))) {
			name =
				"ATTR"
					+ itemID
					+ "_"
					+ UIUtils.replaceSlash(attribute.getName());
		} else {
			name = attribute.getName();
		}

		String[] attribRequestVals = portletRequest.getParameterValues(name);

		if (attribRequestVals == null) {
			// check if the min. occurrence is 0 or not
			if (attribute.getMinimumOccurrence() != 0) {
				// force the user to enter atleast min. occurrence values..
				messageMap.addErrorMessage("MESSAGE_CODE_110", " " + name);
			}
		} else if (
			attribRequestVals.length < attribute.getMinimumOccurrence()) {
			// the values entered is not meeting the min. requirements.
			// force the user to enter atleast min. occurrence values..
			//						messageMap.addErrorMessage("MESSAGE_CODE_110" , new Object[]{name, attribute.getMinimumOccurrence()+""});
			messageMap.addErrorMessage("MESSAGE_CODE_110", " " + name);
		} else {
			for (int i = 0; i < attribRequestVals.length; i++) {
				if (UIUtils.isEmpty(attribRequestVals[i])
					&& (i < attribute.getMinimumOccurrence())) {
					messageMap.addErrorMessage(
						"MESSAGE_CODE_110",
						" " + name + " " + (i + 1));
				}
			}
		}
	}

	/**
	 * The XML generation capability is externilzed to CatalogItemXMLGenerator
	 *
	 */
	public String toXML() {
		return null;
	}

	private void prepareAttributesMap() {
		if (attributes == null) {
			return;
		}

		attributesMap = new HashMap();

		for (int i = 0; i < attributes.length; ++i) {
			attributesMap.put(attributes[i].getName(), attributes[i]);
		}
	}

	public HashMap getAttributeMap() {
		return attributesMap;
	}

	public String getCatalogVersion() {
		return ctgVersion;
	}

	public String getParentCategoryID() {
		return prentCtgyID;
	}

	public void setCatalogVersion(String ctgVersion) {
		this.ctgVersion = ctgVersion;
	}

	public void setParentCategoryID(String parCtgyID) {
		this.prentCtgyID = parCtgyID;
	}

	public void printMe() {
		System.out.println(
			"******************** ITEM DETAILS ****************");
		System.out.println("ParentCategory::" + prentCtgyID);
		System.out.println("CatalogName::" + catalogID);
		System.out.println("ItemID::" + itemID);

		System.out.println("Printing the attributes list");

		for (int i = 0; i < attributes.length; ++i) {
			if (attributes[i] != null) {
				attributes[i].printMe();
			}
		}

		System.out.println(
			"******************** ///ITEM DETAILS ////****************");
	}

	public Attribute getPrimaryKeyAttr(Attribute[] attrArry) {
		if (primaryKeyAttr == null) {
			return preparePrimaryKeyAttr(attrArry);
		} else {
			return primaryKeyAttr;
		}
	}

	public Attribute preparePrimaryKeyAttr(Attribute[] attrArry) {
		for (int k = 0; k < attrArry.length; k++) {
			Attribute attrTemp = attrArry[k];

			if (attrTemp.isPrimaryKey()) {
				primaryKeyAttr = attrTemp;

				break;
			}
		}

		return primaryKeyAttr;
	}

	/**
	 * prepare map of all attributes available including the map inside group attributes
	 * @param attributesParam
	 * @return
	 */
	private HashMap prepareFlatAttributeMap(Attribute[] attributesParam) {
		for (int i = 0; i < attributesParam.length; i++) {
			if (attributesParam[i] != null) {
				if (attributesParam[i].getType() == AttributeTypes.GROUP) {
					flatAttributeMap.put(attributesParam[i].getName(),
						attributesParam[i]);
					prepareFlatAttributeMap(((GroupAttribute) attributesParam[i]).getAttributes());
				} else {
					//if (attributesParam[i].isLocalized() && attributesParam[i].getLocalizedValues().size() > 0){
                    
					if (attributesParam[i].isLocalized()) {
						flatAttributeMap.put(attributesParam[i].getName(),
													attributesParam[i]);
						Attribute[] attrTempArray = new Attribute[attributesParam[i].getLocalizedValues().size()];
						for (int j = 0; j < attributesParam[i].getLocalizedValues().size(); j++) {
							attrTempArray[j] = (Attribute)attributesParam[i].getLocalizedValues().get(j);
						}
						prepareFlatAttributeMap(attrTempArray);
					} else {
						flatAttributeMap.put(attributesParam[i].getName(),
							attributesParam[i]);
					}
				}
			}
		}

		return flatAttributeMap;
	}

	/**
	 * @return
	 */
	public HashMap getFlatAttributeMap() {
		return flatAttributeMap;
	}

	/**
	 * @see com.ibm.ccd.ui.dataobjects.items.ICatalogItem#getCollaborationArea()
	 */
	public CollaborationArea getCollaborationArea() {
		return collabArea;
	}

	/**
	 * @see com.ibm.ccd.ui.dataobjects.items.ICatalogItem#setCollaborationArea(com.ibm.ccd.ui.dataobjects.workflow.CollaborationArea)
	 */
	public void setCollaborationArea(CollaborationArea collabArea) {
		this.collabArea = collabArea;
	}

	public MessageMap updateProperty(
		PortletRequest portletRequest,
		String attribName) {
		MessageMap messageMap = new MessageMap();
		Attribute attribute = (Attribute) flatAttributeMap.get(attribName);
		updateAttribProperty(attribute, portletRequest, messageMap);

		return messageMap;
	}

	public MessageMap updateProperties(PortletRequest portletRequest) {
		MessageMap messageMap = new MessageMap();

		for (int i = 0; i < attributes.length; ++i) {
			Attribute attribute = attributes[i];

			if (attribute.getType() == AttributeTypes.SEQUENCE || ! attribute.isEditable()) {
				continue;
			}
			updateAttribProperty(attributes[i], portletRequest, messageMap);
		}

		return messageMap;
	}

	public void updateAttribProperty(
		Attribute attribute,
		PortletRequest portletRequest,
		MessageMap messageMap) {
		String name = null;
		String value = null;
		String path = attribute.getPath();

		if (path != null) {
			int pathIndex = path.lastIndexOf('_');

			if (pathIndex != -1) {
				path = path.substring(0, pathIndex);
			}
		} else {
			//Check for a Localized attribute. The parent/Localized attribute is always with a path=null, 
			//therefore recurse through the localized values that it holds
			if(attribute.isLocalized()){
				for (int i = 0; i < attribute.getLocalizedValues().size(); i++) {
					updateAttribProperty(
					(Attribute)attribute.getLocalizedValues().get(i),
					portletRequest,
					messageMap);
				}
			}
			return;
		}

		int currentOcc = attribute.getCurrOccurrence();

		switch (attribute.getType()) {
			case AttributeTypes.STRING :
				{
					StringAttribute stringAttribute =
						(StringAttribute) attribute;

					stringAttribute.removeValues();

					if (stringAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							stringAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						stringAttribute.setValue(value);
					}
					break;
				}

			case AttributeTypes.PASSWORD :
				{
					PasswordAttribute passwordAttribute =
						(PasswordAttribute) attribute;
					passwordAttribute.removeValues();

					if (passwordAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							passwordAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						passwordAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.CURRENCY :
				{
					CurrencyAttribute currencyAttribute =
						(CurrencyAttribute) attribute;

					currencyAttribute.removeValues();

					if (currencyAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							currencyAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						currencyAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.DATE :
				{
					DateAttribute dateAttribute = (DateAttribute) attribute;
					dateAttribute.removeValues();

					if (dateAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							dateAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						dateAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.INTEGER :
				{
					IntegerAttribute integerAttribute =
						(IntegerAttribute) attribute;


					if (integerAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							if (!UIUtils.isEmpty(value)) {
							    try{
									value =
										UIUtils.parseInteger(
											value,
											portletRequest.getLocale());
									
									integerAttribute.removeValue(i);
									integerAttribute.setValue(i,
										UIUtils.getIntValue(value));
								} catch (Exception p) {
									p.printStackTrace();
								}
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						if (!UIUtils.isEmpty(value)) {
							try {
                                value =
                                	UIUtils.parseInteger(
                                		value,
                                		portletRequest.getLocale());
                                integerAttribute.removeValues();
                                integerAttribute.setValue(0,
                                	UIUtils.getIntValue(value));
                            } catch (Exception p) {
                                p.printStackTrace();
                            }
						}
					}

					break;
				}

			case AttributeTypes.NUMBER :
				{
					NumberAttribute numberAttribute =
						(NumberAttribute) attribute;


					if (numberAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);

							try {
                                if (!UIUtils.isEmpty(value)) {
                                	value = UIUtils.parseNumber(value,
                                			portletRequest.getLocale());
                                	numberAttribute.removeValue(i);								
                                	numberAttribute.setValue(i,
                                		UIUtils.getDoubleValue(value));
                                }
                            } catch (ParseException e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						try {
                            if (!UIUtils.isEmpty(value)) {
                            	value = UIUtils.parseNumber(value,
                            			portletRequest.getLocale());
                            	numberAttribute.removeValues();
                            	numberAttribute.setValue(0,
                            		UIUtils.getDoubleValue(value));
                            }
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
					}

					break;
				}

			case AttributeTypes.FLAG :
				{
					FlagAttribute flagAttribute = (FlagAttribute) attribute;
					attribute.removeValues();

					if (flagAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);

							if (!UIUtils.isEmpty(value)) {
								flagAttribute.setValue("true");
							} else {
								flagAttribute.setValue("false");
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");

						if (!UIUtils.isEmpty(value)) {
							flagAttribute.setValue("true");
						} else {
							flagAttribute.setValue("false");
						}
					}

					break;
				}
				
			case AttributeTypes.PERIOD :
			{
				PeriodAttribute periodAttribute = (PeriodAttribute) attribute;
				periodAttribute.removeValues();
				
				if(periodAttribute.isMultiValued()) {
					for(int i=0;i<currentOcc;i++) {
						String startDate = portletRequest.getParameter(path + "_" + i+"_"+"V1");
						String endDate = portletRequest.getParameter(path + "_" + i+"_"+"V2");
						periodAttribute.setValue(startDate,endDate);
					}
				}else {
					String startDate = portletRequest.getParameter(path + "_" + "0" +"_"+"V1");
					String endDate = portletRequest.getParameter(path + "_" + "0" +"_"+"V2");
					periodAttribute.setValue(startDate,endDate);
				}
		
				break;
			}
			case AttributeTypes.URL :
				{
					URLAttribute urlAttribute = (URLAttribute) attribute;
					urlAttribute.removeValues();

					if (urlAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);

							try {
								if (!UIUtils.isEmpty(value)) {
									urlAttribute.setValue(new URL(value));
								}else {
									urlAttribute.setValue(null);
								}
							} catch (MalformedURLException e) {
								e.printStackTrace();
								urlAttribute.setValue(null);
								System.out.println("URL value " + value);
								
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						try {
							if(!UIUtils.isEmpty(value)) {
								urlAttribute.setValue(new URL(value));								
							} else {
								urlAttribute.setValue(null);
							}
						} catch (MalformedURLException e) {
							e.printStackTrace();
							urlAttribute.setValue(null);
						}
					}

					break;
				}

			case AttributeTypes.IMAGE_URL :
				{
					ImageURLAttribute imageURLAttribute =
						(ImageURLAttribute) attribute;
					imageURLAttribute.removeValues();

					if (imageURLAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							
							try {
								if (!UIUtils.isEmpty(value)) {
									imageURLAttribute.setValue(new URL(value));
								}else {
									imageURLAttribute.setValue(null);
								}
							} catch (MalformedURLException e) {
								e.printStackTrace();
								imageURLAttribute.setValue(null);
								System.out.println("Image URL value " + value);
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						try {
							if(!UIUtils.isEmpty(value)) {
								imageURLAttribute.setValue(new URL(value));	
							} else {
								imageURLAttribute.setValue(null);
							}
						} catch (MalformedURLException e) {
							e.printStackTrace();
							imageURLAttribute.setValue(null);							
						}
					}

					break;
				}

			case AttributeTypes.THUMB_NAIL_IMAGE_URL :
				{
					ThumbnailImageURLAttribute thumbnailImageURLAttribute =
						(ThumbnailImageURLAttribute) attribute;
					thumbnailImageURLAttribute.removeValues();

					if (thumbnailImageURLAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							
							try {
								if (!UIUtils.isEmpty(value)) {
									thumbnailImageURLAttribute.setValue(
										new URL(value));
								}else {
									thumbnailImageURLAttribute.setValue(null);									
								}
							} catch (MalformedURLException e) {
								e.printStackTrace();
								System.out.println(
									"Thum Image URL value " + value);
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						try {
							if(!UIUtils.isEmpty(value)) {
								thumbnailImageURLAttribute.setValue(new URL(value));	
							}else {
								thumbnailImageURLAttribute.setValue(null);
							}
						} catch (MalformedURLException e) {
							e.printStackTrace();
							thumbnailImageURLAttribute.setValue(null);							
						}
					}

					break;
				}

			case AttributeTypes.BINARY :
				{
					BinaryAttribute binaryAttrib = (BinaryAttribute) attribute;
					FileUploadResource[] files = null;

					if (binaryAttrib.isMultiValued()) {
						files = binaryAttrib.getValues();

						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);

							if ((files[i] != null)
								&& ((files[i].getName() == null)
									|| !files[i].getName().equals(value))) {
								// set server details only when user update with new file attachment
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(value);
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);

						FileUploadResource file = binaryAttrib.getValue();

						if ((file != null)
							&& ((file.getName() == null)
								|| !file.getName().equals(value))) {
							// set server details only when user update with new file attachment
							file.setDirectory("");
							file.setName("");
							file.setPassword("");
							file.setServer("");
							file.setType("");
							file.setUserID("");
							file.setName(value);
						}
					}

					break;
				}

			case AttributeTypes.IMAGE :
				{
					ImageAttribute imageAttribute = (ImageAttribute) attribute;
					FileUploadResource[] files = null;

					if (imageAttribute.isMultiValued()) {
						files = imageAttribute.getValues();

						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);

							if ((files[i] != null)
								&& ((files[i].getName() == null)
									|| !files[i].getName().equals(value))) {
								// set server details only when user update with new file attachment
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(value);
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);

						FileUploadResource file = imageAttribute.getValue();

						if ((file != null)
							&& ((file.getName() == null)
								|| !file.getName().equals(value))) {
							// set server details only when user update with new file attachment
							file.setDirectory("");
							file.setName("");
							file.setPassword("");
							file.setServer("");
							file.setType("");
							file.setUserID("");
							file.setName(value);
						}
					}

					break;
				}

			case AttributeTypes.THUMB_NAIL_IMAGE :
				{
					ThumbnailImageAttribute thumbnailImageAttribute =
						(ThumbnailImageAttribute) attribute;
					FileUploadResource[] files = null;

					if (thumbnailImageAttribute.isMultiValued()) {
						files = thumbnailImageAttribute.getValues();

						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							
							if ((files[i] != null)
								&& ((files[i].getName() == null)
									|| !files[i].getName().equals(value))) {
								// set server details only when user update with new file attachment
								files[i].setDirectory("");
								files[i].setName("");
								files[i].setPassword("");
								files[i].setServer("");
								files[i].setType("");
								files[i].setUserID("");
								files[i].setName(value);
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						
						FileUploadResource file =
							thumbnailImageAttribute.getValue();

						if ((file != null)
							&& ((file.getName() == null)
								|| !file.getName().equals(value))) {
							// set server details only when user update with new file attachment
							file.setDirectory("");
							file.setName("");
							file.setPassword("");
							file.setServer("");
							file.setType("");
							file.setUserID("");
							file.setName(value);
						}
					}

					break;
				}

			case AttributeTypes.STRING_ENUMERATION :
				{
					StringEnumAttribute stringEnumAttribute =
						(StringEnumAttribute) attribute;
					stringEnumAttribute.removeValues();

					if (stringEnumAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							stringEnumAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						stringEnumAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.NUMBER_ENUMERATION :
				{
					NumberEnumAttribute numberEnumAttribute =
						(NumberEnumAttribute) attribute;
					numberEnumAttribute.removeValues();

					if (numberEnumAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);

							try {
								if(!(value !=null && value.equals("-1"))){
									numberEnumAttribute.setValue(
										new Double(value).doubleValue());
								}
							} catch (Exception e) {
								e.printStackTrace();
							}
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");

						try {
							if(!(value !=null && value.equals("-1"))){
								numberEnumAttribute.setValue(
									new Double(value).doubleValue());
							}
						} catch (Exception e) {
							e.printStackTrace();
						}
					}

					break;
				}

			case AttributeTypes.TIMEZONE :
				{
					TimeZoneAttribute timeZoneAttribute =
						(TimeZoneAttribute) attribute;
					timeZoneAttribute.removeValues();

					if (timeZoneAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							timeZoneAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						timeZoneAttribute.setValue(value);
					}

					break;
				}

			case AttributeTypes.UNSUPPORTED :
				break;

			case AttributeTypes.GROUP :
				{
					GroupAttribute groupAttrib = (GroupAttribute) attribute;
					Attribute[] subGroupAttrib = groupAttrib.getAttributes();

					for (int i = 0; i < currentOcc; i++) {
						subGroupAttrib = groupAttrib.getValue(i);

						for (int j = 0; j < subGroupAttrib.length; j++) {
							updateAttribProperty(
								subGroupAttrib[j],
								portletRequest,
								messageMap);
						}
					}

					break;
				}

			case AttributeTypes.RELATIONSHIP :
				{
					String catalogName = null;
					String key = null;
					String itemId = null;
					NVPair pair = null;
					RelationshipAttribute relationshipAttribute =
						(RelationshipAttribute) attribute;
					relationshipAttribute.removeValues();

					if (relationshipAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							catalogName =
								portletRequest.getParameter(
									path + "_" + i + "_V1");
							itemId =
								portletRequest.getParameter(
									path + "_" + i + "_V2");
							catalogName = UIUtils.unescapeForHTML(catalogName);
							itemId = UIUtils.unescapeForHTML(itemId);		
							pair = new NVPair(catalogName, itemId);
							relationshipAttribute.setValue(pair);
						}
					} else {
						catalogName =
							portletRequest.getParameter(path + "_" + 0 + "_V1");
						itemId =
							portletRequest.getParameter(path + "_" + 0 + "_V2");
						catalogName = UIUtils.unescapeForHTML(catalogName);
						itemId = UIUtils.unescapeForHTML(itemId);		
						pair = new NVPair(catalogName, itemId);
						relationshipAttribute.setValue(pair);
					}

					break;
				}

			case AttributeTypes.LOOKUP_TABLE :
				{
					LookupTableAttribute lookupTableAttribute =
						(LookupTableAttribute) attribute;
					lookupTableAttribute.removeValues();

					if (lookupTableAttribute.isMultiValued()) {
						for (int i = 0; i < currentOcc; i++) {
							value = portletRequest.getParameter(path + "_" + i);
							value = UIUtils.unescapeForHTML(value);
							lookupTableAttribute.setValue(value);
						}
					} else {
						value = portletRequest.getParameter(path + "_" + "0");
						value = UIUtils.unescapeForHTML(value);
						lookupTableAttribute.setValue(value);
					}

					break;
				}
		}
	}

	public Attribute getAttributeFromhashValue(String hashValue) {
		HashMap flatAttrib = getFlatAttributeMap();
		Iterator keyIter = flatAttrib.keySet().iterator();
		Attribute attrib = null;

		while (keyIter.hasNext()) {
			attrib = (Attribute) flatAttrib.get(keyIter.next());

			if (attrib.getHashName().equals(hashValue)) {
				return attrib;
			}
		}

		return attrib;
	}
	public String getAttributeNameFromhashValue(String hashValue) {
		Attribute attrib = getAttributeFromhashValue(hashValue);
		if (attrib != null) {
			return attrib.getName();
		} else {
			return null;
		}
	}
	/**
	 *
	 * @param attribPath is GroupID/GroupOcc/AttribID/AttribOcc
	 * @return
	 */
	public Attribute getAttributeFromPath(String attribPath) {
		if (UIUtils.isEmpty(getGroupAttributeID(attribPath))) {
			String attribID = getAttributeID(attribPath);

			return getAttributeFromhashValue(attribID);
		} else {
			String groupID = getGroupAttributeID(attribPath);
			String groupOcc = getGroupAttributeOcc(attribPath);
			String attribID = getAttributeID(attribPath);
			GroupAttribute groupAttrib =
				(GroupAttribute) getAttributeFromhashValue(groupID);
			Attribute[] attribs =
				groupAttrib.getValue(Integer.parseInt(groupOcc));

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i].getName().equals(attribID)) {
					return attribs[i];
				}
			}
			return null;
		}
	}

	/**
	 * attribPath should contain both groupId and attrubuteId
	 * @param attribPath
	 * @return
	 */
	public Attribute getAttributeFromGroupPath(String attribPath)
		throws Exception {
		return getAttributeFromGroupPath(attributes, attribPath);
	}

	/**
	 * attribPath should contain both groupId and attributeId 
	 * @param attributes
	 * @param attribPath
	 * @return
	 */
	public Attribute getAttributeFromGroupPath(
		Attribute[] attribs,
		String attribPath)
		throws Exception {
		StringTokenizer tokenizer = new StringTokenizer(attribPath, "_");

		//contains nestedGroup
		String groupId = null;
		String groupOcc = null;

		String attribId = null;
		String attribOcc = null;
		String truncPath = null;
		if (tokenizer.countTokens() > 4) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			truncPath = groupId + "_" + groupOcc + "_";
			truncPath = attribPath.substring(truncPath.length());

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));
					return getAttributeFromGroupPath(subAttribs, truncPath);
				}
			}
		} else if (tokenizer.countTokens() > 2) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			attribId = tokenizer.nextToken();
			attribOcc = tokenizer.nextToken();

			for (int i = 0; i < attribs.length; i++) {
				
				if (attribs[i]
					.getName()
					.equals(getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));

					for (int j = 0; j < subAttribs.length; j++) {
						if (subAttribs[j]
							.getName()
							.equals(
								getAttributeNameFromhashValue(attribId))) {
							return subAttribs[j];
						}
					}

				}
			}
		} else {
			attribId = tokenizer.nextToken();
			attribOcc = tokenizer.nextToken();
			//Check for a Localized attribute. The parent/Localized attribute is always with a path=null, 
			//therefore recurse through the localized values that it holds

			for (int i = 0; i < attributes.length; i++) {
				
				if(attributes[i].isLocalized()){
					for (int k = 0; k < attributes[i].getLocalizedValues().size(); k++) {
						Attribute localizedAttribute = (Attribute) attributes[i].getLocalizedValues().get(k);
						if (localizedAttribute
							.getName()
							.equals(
								getAttributeNameFromhashValue(attribId))) {
							return localizedAttribute;
						}
					} // End of for loop getLocalizedvalues array
				}else{  // End isLocalized check
					if (attributes[i]
						.getName()
						.equals(
							getAttributeNameFromhashValue(attribId))) {
						return attributes[i];
					}
				}
			}// End attributes array

		}
		return null;

	}
	/**
	 * 
	 * @param attribPath is GroupID_GroupOcc_GroupID_GroupOcc_GroupID_GroupOcc
	 * @return
	 */
	public Attribute[] getAttributesFromGroupPath(String attribPath)
		throws Exception {
		return getAttributesFromGroupPath(attributes, attribPath);
	}

	/**
	 * 
	 * @param attribPath is GroupID_GroupOcc_GroupID_GroupOcc_GroupID_GroupOcc
	 * @return
	 */
	public Attribute[] getAttributesFromGroupPath(
		Attribute[] attribs,
		String attribPath)
		throws Exception {

		StringTokenizer tokenizer = new StringTokenizer(attribPath, "_");

		//contains nestedGroup
		String groupId = null;
		String groupOcc = null;
		String truncPath = null;
		if (tokenizer.countTokens() > 2) {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc
			truncPath = groupId + "_" + groupOcc + "_";
			truncPath = attribPath.substring(truncPath.length());


			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));
					return getAttributesFromGroupPath(subAttribs, truncPath);
				}
			}
		} else {
			groupId = tokenizer.nextToken(); // GroupId
			groupOcc = tokenizer.nextToken(); // GroupOcc

			for (int i = 0; i < attribs.length; i++) {
				if (attribs[i]
					.getName()
					.equals(
						getAttributeNameFromhashValue(groupId))) {
					GroupAttribute groupAttrib = (GroupAttribute) attribs[i];
					if (!groupAttrib.hasValue(Integer.parseInt(groupOcc))) {
						throw new Exception("Exceeds Group limit " + attribPath);
					}
					Attribute[] subAttribs =
						groupAttrib.getValue(Integer.parseInt(groupOcc));

					return subAttribs;
				}
			}
		}
		return null;
	}
	
	public boolean isValidGroupAttributePath(String attribPath) {
		return true;
	}
	
	public Attribute getGroupAttribute(Attribute attribute) {
		String attribName = attribute.getName();
		int firstIndex = attribName.indexOf('_');
		int lastIndex = attribName.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return null;
		} else {
			attribName = attribName.substring(0, lastIndex);

			return getAttribute(attribName);
		}
	}

	public static String getGroupAttributeID(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return "";
		} else {
			return attribParam.substring(0, firstIndex);
		}
	}

	public static String getGroupAttributeOcc(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return "";
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return attribParam.substring(firstIndex + 1, nextIndex);
		}
	}

	public static String getAttributeID(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return attribParam.substring(0, firstIndex);
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return getAttributeID(attribParam.substring(nextIndex + 1));
		}
	}

	public static String getAttributeOcc(String attribParam) {
		int firstIndex = attribParam.indexOf('_');
		int lastIndex = attribParam.lastIndexOf('_');

		if (firstIndex == lastIndex) {
			return attribParam.substring(firstIndex + 1);
		} else {
			int nextIndex = attribParam.indexOf('_', firstIndex + 1);

			return getAttributeOcc(attribParam.substring(nextIndex + 1));
		}
	}
	
	/**
	 * @return HashMap return the attribute Map for Item Download
	 */
	public HashMap getDownloadAttributeMap() {
		downloadAttributeMap = new HashMap();
		return getDownloadAttributeMap(attributes);
	}

	/**
	 * @return HashMap return the attribute Map for Item Download
	 */
	
	private HashMap getDownloadAttributeMap(Attribute[] attributes) {
		
		for (int i = 0; i < attributes.length; i++) 
		{
			if (attributes[i] != null) 
			{
				if (attributes[i].getType() == AttributeTypes.GROUP) 
				{
					getDownloadAttributeMap(((GroupAttribute) attributes[i]).getAttributes());
				} else 
				{
					if(attributes[i].isLocalized()) {
						Attribute[] attrTempArray = new Attribute[attributes[i].getLocalizedValues().size()];
						for (int j = 0; j < attributes[i].getLocalizedValues().size(); j++) {
							attrTempArray[j] = (Attribute)attributes[i].getLocalizedValues().get(j);
						}
						getDownloadAttributeMap(attrTempArray);
					} else 
					{
						downloadAttributeMap.put(attributes[i].getName(),attributes[i]);	
					}
				}
			}
		}

		return downloadAttributeMap;
		
	}	
	
}
