
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package com.ibm.ccd.ui.mediator;

import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;
import javax.xml.soap.AttachmentPart;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.utils.Messages;

import com.ibm.ccd.ui.dataobjects.Attachment;
import com.ibm.ccd.ui.util.IReference;


/**
 * Provides the support for the SOAP calls made on WPC
 * Uses Axis API and supports attacments(in doc mode), rpc and doc. modes
 */
public class SoapInvoker {
    public static final String COMP_NAME = "SoapInvoker";

    /**
     * SOAP call made over RPC
     * @param url - EndPoint address
     * @param opNamespace - Operation SOAP namespace
     * @param opName - Operation Name
     * @param params - Operation parameters
     */
    public static Object invokeSoapWithRPC(String url, String opNamespace,
        String opName, Object[] params)
        throws ServiceException, MalformedURLException, RemoteException {
        return invokeSoapWithRPC(url, opNamespace, opName, params, null);
    }

    /**
            * SOAP call made over RPC
            * @param url - EndPoint address
            * @param opNamespace - Operation SOAP namespace
            * @param opName - Operation Name
            * @param params - Operation parameters
            * @param paramNames - parameter names..
            */
    private static Object invokeSoapWithRPC(String url, String opNamespace,
        String opName, Object[] params, Object[] paramNames)
        throws ServiceException, MalformedURLException, RemoteException {
        Service service = new Service();
        Call call = (Call) service.createCall();
        call.setTargetEndpointAddress(new java.net.URL(url));
        call.setOperationName(new QName(opNamespace, opName));

        if (paramNames != null) {
            for (int i = 0; i < params.length; ++i) {
                call.addParameter((String) paramNames[i],
                    call.getTypeMapping().getTypeQName(params[i].getClass()),
                    javax.xml.rpc.ParameterMode.IN);
            }

            call.setReturnType(org.apache.axis.Constants.XSD_STRING);
        }

        return call.invoke(params);
    }

    /**
     * SOAP call mode using DOC. literal mode and supports attachements
     *
     */
    public static SoapResponse invokeSoapWithMOM(String url, String requestMsg,
        Attachment attachment)
        throws ServiceException, MalformedURLException, AxisFault {
        Service service = new Service();
        Call call = (Call) service.createCall();
        Object timeoutValue = ConfigManager.getInstance().getFrameworkConfig()
                                           .get(IReference.AXIS_CLIENT_TIMEOUT_MILLISECONDS);

        if (timeoutValue != null) {
            try {
                Integer timeout = null;
                timeout = new Integer((String) timeoutValue);
                call.getMessageContext().setTimeout(timeout.intValue());
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
        }

        call.setTargetEndpointAddress(new java.net.URL(url));

        return invokeWithMOM(call, requestMsg, attachment);
    }

    /**
      * SOAP call mode using DOC. literal mode and supports attachements
      *
      */
    private static SoapResponse invokeWithMOM(Call call, String requestMsg,
        Attachment attachment) throws AxisFault {
        SOAPEnvelope env = new SOAPEnvelope(call.getMessageContext()
                                                .getSOAPConstants(),
                call.getMessageContext().getSchemaVersion());
        SOAPBodyElement elmt = null;

        try {
            elmt = new SOAPBodyElement(new ByteArrayInputStream(
                        requestMsg.getBytes("UTF-8")));
            env.addBodyElement(elmt);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();

            return null;
        }

        Message msg = new Message(env);

        if (attachment != null) {
            AttachmentPart inAttchPart = msg.createAttachmentPart();
            inAttchPart.setContent(attachment.getObjectForUpload(),
                attachment.getMimeType());
            msg.addAttachmentPart(inAttchPart);
        }

        call.setRequestMessage(msg);
        call.invoke();

        msg = call.getMessageContext().getResponseMessage();

        if (msg == null) {
            if (Call.FAULT_ON_NO_RESPONSE) {
                throw new AxisFault(Messages.getMessage("nullResponse00"));
            } else {
                return null;
            }
        }

        //used for the attachments
        ArrayList attachmentList = new ArrayList();
        Iterator it = call.getMessageContext().getMessage().getAttachments();

        while (it.hasNext()) {
            /*AttachmentPart attachmentPart = (AttachmentPart) it.next();
            attachment.getObjectForUpload();
            System.out.println(attachmentPart.getContentId());
            System.out.println(attachmentPart.getContentType());
            */
            attachmentList.add(it.next());
        }

        env = msg.getSOAPEnvelope();

        Vector v = env.getBodyElements();
        StringBuffer buff = new StringBuffer();

        for (int i = 0; i < v.size(); i++) {
            buff.append(((SOAPBodyElement) v.get(i)).toString());
        }

        return new SoapResponse(buff.toString(), attachmentList);
    }
}
