
/*
* Licensed Materials: Property of IBM
* "Restricted Materials of IBM"
* 5724-I68.
* (C) Copyright IBM Corp. 2000, 2005 - All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package com.ibm.ccd.ui.tags.element;

import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.PortletResponse;
import org.apache.jetspeed.portlet.PortletURI;

import com.ibm.ccd.ui.dataobjects.attributes.Attribute;
import com.ibm.ccd.ui.dataobjects.attributes.DateAttribute;
import com.ibm.ccd.ui.util.UIUtils;

public class DateElement extends BaseElement {

	public DateElement(Attribute attribute, PortletURI portletURI) {
		super(attribute, portletURI);
	}

	/**
	 * @see com.ibm.ccd.wpc.tags.BaseElement#displayField(long, String)
	 */
	public String displayField(long id, Object value) {
		StringBuffer buffer = new StringBuffer();
		addHTMLProperty(HTMLVALUE, (String) value);

		String path = getAttributeID(String.valueOf(id));
		attribute.setPath(path);
		addHTMLProperty(HTMLID, path);

		addHTMLProperty(HTMLNAME, path);

		buffer.append(HTML_ATTRIBUTE_START);

		if (isEditable()) {
			renderCalendarScript(buffer, path, String.valueOf(value));

			if (attribute.isMultiValued()) {
				buffer.append(displayPlusMinus(id));
			}
		} else {
			buffer.append(parseWPCDate((String) value));

		}

		buffer.append(HTML_ATTRIBUTE_END);

		return buffer.toString();
	}

	public String renderElement() {
		StringBuffer buffer = new StringBuffer();

		if (attribute.isMultiValued()) {
			for (int i = 0; i < attribute.getCurrOccurrence(); i++) {
				buffer.append(displayLabel());
				buffer.append(displayField(i, getStringValue(attribute, i)));
			}
		} else {
			buffer.append(displayLabel());
			buffer.append(displayField(0, getStringValue(attribute, 0)));
		}

		return buffer.toString();
	}

	/**
	 * @param htmlCode
	 * @param calendarId
	 * @param value
	 */
	protected void renderCalendarScript(
		StringBuffer htmlCode,
		String calendarId,
		String value) {
		StringBuffer buffer = new StringBuffer("");
		String calendarId1 = calendarId;

		calendarId = calendarId.substring(calendarId.lastIndexOf('/') + 1);

		for (int i = 0; i < calendarId.length(); i++) {
			if (Character.isSpaceChar(calendarId.charAt(i))) {
				continue;
			}

			if (calendarId.charAt(i) != '/') {
				buffer.append(calendarId.charAt(i));
			}
		}
		
		int occ = 0;
		int occ_Index = calendarId.lastIndexOf('_');
		if(occ_Index != -1) {
			occ = Integer.parseInt(calendarId.substring(occ_Index+1)); 
		}
		
		String encodeurl =
			(
				(PortletResponse) pageContext.getAttribute(
					"portletResponse")).encodeURL(
				"/");

		PortletRequest portletRequest =
			(PortletRequest) pageContext.getAttribute("portletRequest");

		String fieldValue = "";
		
		if(attribute.getValueList() != null && attribute.getValueList().size() > occ) {
			Object obj = attribute.getValueList().get(occ);
			if(obj != null) {
				fieldValue = (String) obj;	
			}
		}

		htmlCode.append("<script language=\"JavaScript\"> ");

		if (!UIUtils.isEmpty(fieldValue)) {
			htmlCode.append(
				"document.write(placeCalendar('"
					+ buffer
					+ "','"
					+ encodeurl
					+ "','"
					+ calendarId1
					+ "','"
					+ parseWPCDate(fieldValue)
					+ "'));");
		} else {
			htmlCode.append(
				"document.write(placeCalendar('"
					+ buffer
					+ "','"
					+ encodeurl
					+ "','"
					+ calendarId1
					+ "','"
					+ fieldValue
					+ "'));");
		}
		htmlCode.append("</script>");

		htmlCode.append(
			"<input type=\"hidden\" id=\""
				+ calendarId1
				+ "\" name=\""
				+ calendarId1
				+ "\" value=\""
				+ fieldValue
				+ "\" >");
	}

	/**
	 * @param attribute
	 * @param index
	 * @return
	 */
	protected String getStringValue(Attribute attribute, int index) {
		DateAttribute dateAttrib = (DateAttribute) attribute;

		if (dateAttrib.getCurrentOccurrence() > 0) {
			if (dateAttrib.isMultiValued()) {
				String[] values = dateAttrib.getValues();

				return (values[index] != null) ? values[index] : "";
			} else {
				return (dateAttrib.getValue() != null)
					? dateAttrib.getValue()
					: "";
			}
		} else {
			return (dateAttrib.getDefaultValue() != null)
				? dateAttrib.getDefaultValue()
				: "";
		}
	}

	/**
	 * Converts WPCDate String format yyyy-mm-dd HH:MM:SS to custom format 
	 * specified in wpclabels.properties  
	 * @param date_str
	 * @return
	 */
	protected String parseWPCDate(String date_str) {
		PortletRequest portletRequest =
			(PortletRequest) pageContext.getAttribute("portletRequest");
		Locale locale = portletRequest.getLocale();

		ResourceBundle bundle =
			ResourceBundle.getBundle(
				"com.ibm.ccd.ui.nls.labels.wpclabels",
				locale);
		String customFormat = bundle.getString("CALENDAR.DATE.FORMAT");

		int yy = 0, mm = 0, dd = 0, HH = 0, MM = 0, SS = 0;

		String WPC_DATE_FORMAT = bundle.getString("WPC.DATE.FORMAT");

		try{
			
		int i_yy = WPC_DATE_FORMAT.indexOf("yyyy");
		if (i_yy != -1) {
			yy = Integer.parseInt(date_str.substring(i_yy, i_yy + 4));
		}

		int i_mm = WPC_DATE_FORMAT.indexOf("mm");
		if (i_mm != -1) {
			mm = Integer.parseInt(date_str.substring(i_mm, i_mm + 2));
		}

		int i_dd = WPC_DATE_FORMAT.indexOf("dd");
		if (i_dd != -1) {
			dd = Integer.parseInt(date_str.substring(i_dd, i_dd + 2));
		}

		int i_HH = WPC_DATE_FORMAT.indexOf("HH");
		if (i_HH != -1) {
			HH = Integer.parseInt(date_str.substring(i_HH, i_HH + 2));
		}

		int i_MM = WPC_DATE_FORMAT.indexOf("MM");
		if (i_MM != -1) {
			MM = Integer.parseInt(date_str.substring(i_MM, i_MM + 2));
		}

		int i_SS = WPC_DATE_FORMAT.indexOf("SS");
		if (i_SS != -1) {
			SS = Integer.parseInt(date_str.substring(i_SS, i_SS + 2));
		}

		customFormat = replace(customFormat, "dd", "<e>");
		customFormat = replace(customFormat, "d", "<d>");
		customFormat = replace(customFormat, "<e>", padZero(dd));
		customFormat = replace(customFormat, "<d>", String.valueOf(dd));
		customFormat = replace(customFormat, "mm", "<n>");
		customFormat = replace(customFormat, "m", "<m>");
		customFormat = replace(customFormat, "<m>", String.valueOf(mm));
		customFormat = replace(customFormat, "<n>", padZero(mm));
		customFormat = replace(customFormat, "yyyy", "<4y>");
		customFormat = replace(customFormat, "yy", "<2y>");
		customFormat = replace(customFormat, "<4y>", padZero(yy));
		customFormat = replace(customFormat, "<2y>", padZero(yy));

		customFormat = replace(customFormat, "HH", padZero(HH));
		customFormat = replace(customFormat, "MM", padZero(MM));
		customFormat = replace(customFormat, "SS", padZero(SS));
		}catch(Exception e) {
			//return empty string when exception occurs while parsing 
			return "";
		}
		return customFormat;
	}

	/**
	 * searches the original string for the search String and replaces 
	 * it with replace String.   
	 * @param original
	 * @param searchStr
	 * @param replaceStr
	 * @return
	 */
	public String replace(
		String original,
		String searchStr,
		String replaceStr) {

		StringBuffer buffer = new StringBuffer(original);
		int index = 0;

		while ((index = original.indexOf(searchStr, index)) != -1) {
			buffer.replace(index, index + searchStr.length(), replaceStr);
			index++;
		}
		return buffer.toString();
	}

	/**
	 * pad Zero if the number is less than 9
	 * @param num
	 * @return
	 */
	public String padZero(int num) {
		return (num < 10) ? "0" + num : num + "";
	}

}
